@@ -20,7 +20,7 @@ from xarray.core.variable import IndexVariable, Variable
 from xarray.namedarray.utils import is_duck_dask_array
 
 if TYPE_CHECKING:
-    import sys
+    from typing import Self
 
     from numpy.typing import DTypeLike
 
@@ -28,11 +28,6 @@ if TYPE_CHECKING:
     from xarray.core.dataset import Dataset
     from xarray.core.types import CFCalendar
 
-    if sys.version_info >= (3, 11):
-        from typing import Self
-    else:
-        from typing_extensions import Self
-
 
 def _season_from_months(months):
     """Compute season (DJF, MAM, JJA, SON) from month ordinal"""
@@ -211,7 +206,7 @@ def _strftime_through_cftimeindex(values, date_format: str):
     values_as_cftimeindex = CFTimeIndex(duck_array_ops.ravel(values))
 
     field_values = values_as_cftimeindex.strftime(date_format)
-    return field_values.values.reshape(values.shape)
+    return field_values.to_numpy().reshape(values.shape)
 
 
 def _strftime_through_series(values, date_format: str):
@@ -220,7 +215,7 @@ def _strftime_through_series(values, date_format: str):
     """
     values_as_series = pd.Series(duck_array_ops.ravel(values), copy=False)
     strs = values_as_series.dt.strftime(date_format)
-    return strs.values.reshape(values.shape)
+    return strs.to_numpy().reshape(values.shape)
 
 
 def _strftime(values, date_format):
